@@ -31,6 +31,12 @@
 
 #include <libcaja-extension/caja-extension-types.h>
 
+#if PY_MAJOR_VERSION >= 3
+#define STRING_FROMSTRING(str)	PyUnicode_FromString(str)
+#else
+#define STRING_FROMSTRING(str)	PyString_FromString(str)
+#endif
+
 static const GDebugKey caja_python_debug_keys[] = {
 	{"misc", CAJA_PYTHON_DEBUG_MISC},
 };
@@ -145,7 +151,7 @@ caja_python_load_dir (GTypeModule *module,
 				
 				/* sys.path.insert(0, dirname) */
 				sys_path = PySys_GetObject("path");
-				py_path = PyString_FromString(dirname);
+				py_path = STRING_FROMSTRING(dirname);
 				PyList_Insert(sys_path, 0, py_path);
 				Py_DECREF(py_path);
 			}
@@ -157,15 +163,19 @@ caja_python_load_dir (GTypeModule *module,
 static gboolean
 caja_python_init_python (void)
 {
-	PyObject *caja;
+	PyObject *gi, *require_version, *args, *caja;
 	GModule *libpython;
+#if PY_MAJOR_VERSION >= 3
+	wchar_t *argv[] = { L"caja", NULL };
+#else
 	char *argv[] = { "caja", NULL };
+#endif
 
 	if (Py_IsInitialized())
 		return TRUE;
 
-  	debug("g_module_open " PY_LIB_LOC "/libpython" PYTHON_VERSION "." G_MODULE_SUFFIX ".1.0");
-	libpython = g_module_open(PY_LIB_LOC "/libpython" PYTHON_VERSION "." G_MODULE_SUFFIX ".1.0", 0);
+  	debug("g_module_open " PY_LIB_LOC "/libpython" PYTHON_VERSION PYTHON_ABIFLAGS "." G_MODULE_SUFFIX ".1.0");
+	libpython = g_module_open(PY_LIB_LOC "/libpython" PYTHON_VERSION PYTHON_ABIFLAGS "." G_MODULE_SUFFIX ".1.0", 0);
 	if (!libpython)
 		g_warning("g_module_open libpython failed: %s", g_module_error());
 
@@ -186,7 +196,7 @@ caja_python_init_python (void)
 	}
 
 	debug("Sanitize the python search path");
-	PyRun_SimpleString("import sys; sys.path = filter(None, sys.path)");
+	PyRun_SimpleString("import sys; sys.path = list(filter(None, sys.path))");
 	if (PyErr_Occurred())
 	{
 		PyErr_Print();
@@ -204,6 +214,19 @@ caja_python_init_python (void)
 	/* import caja */
 	g_setenv("INSIDE_CAJA_PYTHON", "", FALSE);
 	debug("import caja");
+	gi = PyImport_ImportModule ("gi");
+	if (!gi) {
+		PyErr_Print();
+		return FALSE;
+	}
+
+	require_version = PyObject_GetAttrString (gi, (char *) "require_version");
+	args = PyTuple_Pack (2, PyUnicode_FromString ("Caja"),
+	PyUnicode_FromString ("2.0"));
+	PyObject_CallObject (require_version, args);
+	Py_DECREF (require_version);
+	Py_DECREF (args);
+	Py_DECREF (gi);
 	caja = PyImport_ImportModule("gi.repository.Caja");
 	if (!caja)
 	{
